package com.newrelic.api.agent;

/**
 * A transaction represents a unit of work in an application. It may be a single web request or a scheduled background
 * task. To indicate that a method is the starting point of a transaction, mark it with a {@link Trace} annotation and
 * set {@link Trace#dispatcher()} to true.
 * 
 * @author sdaubin
 * 
 * @see Agent#getTransaction()
 * @see Trace#dispatcher()
 */
public interface Transaction {

    /**
     * Sets the current transaction's name using the given priority. Higher priority levels are given precedence, and if
     * the name is set many times with the same priority, the first call wins unless override is true.
     * 
     * @param namePriority The priority of the new transaction name.
     * @param override Overrides the current transaction name if it has the same priority level (or lower).
     * @param category The type of transaction. This is the second segment of the full transaction metric name.
     * @param parts The category and all of the parts are concatenated together with / characters to create the full
     *        name
     * @return Returns true if the transaction name was successfully changed
     */
    boolean setTransactionName(TransactionNamePriority namePriority, boolean override, String category, String... parts);

    /**
     * Returns true if the transaction name has been set.
     */
    boolean isTransactionNameSet();

    /**
     * Returns this transaction's last tracer.
     * 
     * @deprecated use {@link getTracedMethod}.
     */
    @Deprecated
    TracedMethod getLastTracer();

    /**
     * Returns the TracedMethod enclosing the caller.
     * 
     * @return the TracedMethod enclosing the caller. The return value is null if the caller is not within a
     *         transaction.
     */
    TracedMethod getTracedMethod();

    /**
     * Ignore this transaction so that none of its data is reported to the New Relic service.
     */
    void ignore();

    /**
     * Ignore the current transaction for calculating Apdex score.
     */
    void ignoreApdex();

    /**
     * Make a pending asynchronous activity known to instrumentation, associating the given key with the activity. This
     * method must be called within a Transaction (the "active" Transaction) while running on the thread that creates
     * the asynchronous activity.
     * <p>
     * As a result of this call, the registered activity becomes associated with the active Transaction. The active
     * Transaction will not complete until all associated activities complete. A registered activity may complete in one
     * of three ways:
     * <ul>
     * <li>the activity is started via a call to {@see startAsyncActivity} and proceeds to completion.
     * <li>a call is made to {@see ignoreAsyncActivity} with the same argument.
     * <li>the registration times out.
     * </ul>
     * The timeout period is unspecified but is in the range of several minutes. The timeout behavior is defined in
     * order to prevent unbounded memory growth from occurring as a consequence of ill-behaved applications that do not
     * properly manage their asynchronous activities. Relying on the timeout behavior is strongly discouraged.
     * 
     * @param activityContext the key used by instrumentation to identify this asynchronous activity. The value must be
     *        unique across all asynchronous activities tracked during the life of the current Agent instance.
     * @return true if the registration succeeded else false
     */
    boolean registerAsyncActivity(Object activityContext);

    /**
     * Inform instrumentation that the asynchronous activity identified by the key is starting. This method must be
     * called while running on the thread that executes the asynchronous activity.
     * 
     * @param activityContext the key used by instrumentation to identify this asynchronous activity. The value must
     *        have been previously made known to New Relic by a call to {@see registerAsyncActivity}.
     *        <p>
     *        Implementation note: the activityContext must match the argument passed to {@see registerAsyncActivity} by
     *        identity comparison ("=="). Strings or other values that are derived or computed in the parent activity
     *        and the re-derived or computed in the child do not meet this requirement. In many Java implementations,
     *        string interning is nondeterministic and does not provide a reliable solution.
     * 
     * @return true if the context is recognized and an action is taken. False if the context does not exist.
     */
    boolean startAsyncActivity(Object activityContext);

    /**
     * Inform instrumentation that the asynchronous activity identified by the key should no longer be tracked by
     * instrumentation. This method may be called from any thread.
     * 
     * @param activityContext the key used by instrumentation to identify this asynchronous activity. The value must be
     *        unique across all asynchronous activities tracked during the life of the current Agent instance and must
     *        have been previously made known to New Relic by a call to {@see registerAsyncActivity}.
     * @return true if the context is recognized and an action is taken. False if the context does not exist.
     */
    boolean ignoreAsyncActivity(Object activityContext);

}
